#!/bin/sh

. /etc/utils/shell-utils.sh
DEBUG=0
retVal=0
MOUNTS_CACHE="/ram/mounts.cache"
RDIR_SCRIPT="/home/default/rdir.cgi"

filterSnString()
{
    SERIAL=`echo -n \"$1\" | strings | awk '{ s=$0; gsub(/[^a-zA-Z0-9_]/, "", s); printf s }'`
    if [ "$SERIAL" == "" ]; then
	SERIAL="UNASSIGNED"
    fi
}

# EXECUTION POINT

log "devName='${DEV_NAME}', sn='${SERIAL}', pNum='${PNUM}', vendor='${uVendor}', model='${uModel}', DevPath='${DEVPATH}'"

case $1 in
    add)
	log "add command"
        size=`cat /sys${DEVPATH}/size 2> /dev/null`;
	if [ -z "$size" ]; then
	    log "Size read error!"
	    return
	fi
        size=$(( size * 512 ));

        label=`blkid /dev/${DEV_NAME} | awk '{ i1 = index($0, " LABEL=\""); if(i1 > 0){ s1 = substr($0, i1 + 8); i2 = index(s1, "\""); if (i2 > 0){print substr(s1, 0, i2 - 1);} } }'`;
	fstype=`blkid /dev/${DEV_NAME} | awk '{ i1 = index($0, " TYPE=\""); if(i1 > 0){ s1 = substr($0, i1 + 7); i2 = index(s1, "\""); if (i2 > 0){print substr(s1, 0, i2 - 1);} } }'`;

	if [ "$fstype" == "" ]; then
            log "fstype is empty!"
            exit -1
	fi

	# count "entire disk" as single partition number 0
	if [ "$PNUM" == "" ]; then
	    PNUM="0"
	fi

	# serial number conditioning
	filterSnString $SERIAL

	FULL_NAME="USB-${SERIAL}-${PNUM}"
	FOLDER_TO_CREATE="/media/${FULL_NAME}"
	log "mount point '$FOLDER_TO_CREATE'"

	if [ ! -f "$FOLDER_TO_CREATE" ]; then
	    log "Creating mount point..."
	    mkdir "$FOLDER_TO_CREATE"
	fi

	IS_READ_ONLY="0"
	case $fstype in
	    "ntfs")
		DEV_MODEL=`$RDIR_SCRIPT Model`
		if [ "$DEV_MODEL" != "MAG200" ]; then
		    mount -t tntfs -o rw,iocharset=utf8 $DEVNAME "$FOLDER_TO_CREATE"
		else
		    mount -o ro,iocharset=utf8 $DEVNAME "$FOLDER_TO_CREATE"
		    IS_READ_ONLY="1"
		fi
	    ;;
	    "ext2")
		mount -o rw $DEVNAME "$FOLDER_TO_CREATE"
	    ;;
	    "ext3")
		mount -o rw $DEVNAME "$FOLDER_TO_CREATE"
	    ;;
	    "vfat")
		mount -o rw,iocharset=utf8 $DEVNAME "$FOLDER_TO_CREATE"
	    ;;
	    *)
		mount -o rw $DEVNAME "$FOLDER_TO_CREATE"
	    ;;
	esac
	if [ "$?" != "0" ]; then
	    log "mount error! removing mount point..."
	    rmdir "$FOLDER_TO_CREATE"
	    exit -1
	fi

        freeSize=`df -k | grep /ram/media/$FULL_NAME | awk '{print $4}'`
        freeSize=$((${freeSize}*1024))

	echo "key:${FULL_NAME}:{\"sn\":\"${SERIAL}\",\"vendor\":\"${uVendor}\",\"model\":\"${uModel}\",\"size\":\"${size}\",\"freeSize\":\"${freeSize}\",\"label\":\"${label}\",\"partitionNum\":\"${PNUM}\",\"isReadOnly\":\"${IS_READ_ONLY}\",\"mountPath\":\"${FOLDER_TO_CREATE}\"}" >> $MOUNTS_CACHE
	/usr/share/qt-4.6.0/sendqtevent -a -ks 0x70 -kqt 0x50
    ;;
    remove)
	log "remove command"

	# count "entire disk" as single partition number 0
	if [ -z "$PNUM" ]; then
    	    PNUM="0"
	fi

	# serial number conditioning
	filterSnString $SERIAL

	if [ -d /ram/media/USB-${SERIAL}-${PNUM} ]; then
	    FULL_NAME="USB-${SERIAL}-${PNUM}"
	    FOLDER_TO_DELETE="/media/${FULL_NAME}"
	    log "Unmounting '$FOLDER_TO_DELETE'..."
	    umount "$FOLDER_TO_DELETE"
	    log "Removing '$FOLDER_TO_DELETE'..."
	    rmdir "$FOLDER_TO_DELETE"
	    excludeRaw $MOUNTS_CACHE $FULL_NAME
	    /usr/share/qt-4.6.0/sendqtevent -a -ks 0x71 -kqt 0x51
	fi
    ;;
esac

exit 0
