#!/bin/sh

. /etc/utils/shell-utils.sh
DEBUG=0

RALINK_CFG_FILE=/ram/wifi/configs/RT2870STA.dat
LOCK_FILE=/ram/lock/wifi-init.lock
LOCK_ISACTIVE_FILE=/ram/lock/wifi-isactive.lock
LOCK_OWNED=0
FW_ENV_RESULT=
WIFI_STARTUP_SCRIPT=/etc/init.d/wifi.sh

WIFI_DRIVER_MODULE_FILENAME="rt5370sta.ko"

ntp_static_start()
{
    log "add ntp servers"

    # update NTP servers using DHCP info
    if [ ! -f $NTPCONF ]; then
      cat $NTPCONFDEF > $NTPCONF
      NTP_IP=`fw_printenv ntpurl 2>/dev/null`
      NTP_IP=${NTP_IP#ntpurl=}
      if [ "$NTP_IP" != "" ]; then
        echo server $NTP_IP "#manual" >> $NTPCONF
      fi
    fi
    /etc/init.d/ntp-server restart
}

set_igmp_version()
{
    igmp_conf=`fw_printenv igmp_conf 2>/dev/null`
    igmp_conf=${igmp_conf#igmp_conf=}
    if [ "$igmp_conf" == "V3" ]; then
      echo "3" > /proc/sys/net/ipv4/conf/$1/force_igmp_version
    else
      echo "2" > /proc/sys/net/ipv4/conf/$1/force_igmp_version
    fi
}


_check_lock_and_delete()
{
  if [ $LOCK_OWNED -gt 0 ]; then
    #log "Lock owned. Deleting lock..."
    rm -f $LOCK_FILE
  fi
}

_exit()
{
  _check_lock_and_delete

  exit 0;
}
_error()
{
  _check_lock_and_delete

  exit 1;
}

get_nvram_env()
{
    val=`fw_printenv $1 2>/dev/null`
    FW_ENV_RESULT=${val#*=}
    #FW_ENV_RESULT=`echo -ne $val`
}

read_nvram_env()
{
eval tmpVar=\$$1
#echo $tmpVar
  if [ "$tmpVar" != "" ]; then
    #echo "already set"
    return;
fi
  get_nvram_env $1
  export $1="$FW_ENV_RESULT"
}

_execute()
{
  if [ $DEBUG == "1" ]; then
    log "$1"
  fi

  eval $1
}

_make_ralink_init()
{
  case $wifi_auth in
    open | shared | wep_auto | none)
      #log "Inserting module"
      _execute "insmod /lib/modules/$WIFI_DRIVER_MODULE_FILENAME"
      _execute "sleep 1"
      #log "Configuting \"ra0\"interface"
      if [ "$wifi_int_ip" == "0.0.0.0" ] || [ "$wifi_int_ip" == "" ]; then
        log "Starting DHCP client..."
        _execute "ifconfig ra0 up"
        _execute "sleep 1"
        _execute "iwconfig ra0 essid \"$wifi_ssid\""
        _execute "sleep 1"
        _execute "/etc/init.d/udhcpcd start ra0 &"
        _execute "sleep 1"
        #log "Opps! We dont support DHCP at the moment"
      else
        #log "Configuting static interface settings"
        _execute "ifconfig ra0 $wifi_int_ip netmask $wifi_int_mask"
        _execute "iwconfig ra0 essid \"$wifi_ssid\""
        _execute "sleep 1"
        _execute "route add default gw $wifi_int_gw ra0 metric 1"
	_execute "route add -net 224.0.0.0 netmask 240.0.0.0  dev ra0 metric 1"
        _execute "echo -e \"nameserver \"$wifi_int_dns\" #ra0\" >> /etc/resolv.conf"
        set_igmp_version "ra0"
        ntp_static_start
      fi
    ;;
    wpapsk | wpa2psk)
      #log "Inserting module"
      _execute "insmod /lib/modules/$WIFI_DRIVER_MODULE_FILENAME"
      _execute "sleep 1"
      #log "Configuting \"ra0\"interface"
      if [ "$wifi_int_ip" == "0.0.0.0" ] || [ "$wifi_int_ip" == "" ]; then
        log "Starting DHCP client..."
        _execute "ifconfig ra0 up"
        _execute "sleep 1"
        _execute "/etc/init.d/udhcpcd start ra0 &"
        _execute "sleep 1"
        #log "Opps! We dont support DHCP at the moment"
      else
        #log "Configuting static interface settings"
        _execute "ifconfig ra0 $wifi_int_ip netmask $wifi_int_mask"
        _execute "route add default gw $wifi_int_gw ra0"
	_execute "route add -net 224.0.0.0 netmask 240.0.0.0  dev ra0 metric 1"
        _execute "echo -e \"nameserver \"$wifi_int_dns\" #ra0\" >> /etc/resolv.conf"
        set_igmp_version "ra0"
        ntp_static_start

      fi
    ;;
  esac
}

_make_ralink_deinit()
{
  log "Stopping DHCP client..."
  _execute "/etc/init.d/udhcpcd stop ra0 &"
  _execute "sleep 1"

  #ifconfig ra0 down
  #_execute sleep 1

  log "removing module"
  _execute "rmmod -f $WIFI_DRIVER_MODULE_FILENAME"
  sleep 1
  rm -f $LOCK_ISACTIVE_FILE
}


_make_ralink_cfg()
{
    tmp_ralink_cfg_enc="";
    case $wifi_enc in
	none)
	    tmp_ralink_cfg_enc="EncrypType=NONE"
	;;
	wep)
	    tmp_ralink_cfg_enc="EncrypType=WEP"
	;;
	wep64)
	    tmp_ralink_cfg_enc="EncrypType=WEP"
	;;
	wep128)
	    tmp_ralink_cfg_enc="EncrypType=WEP"
	;;
	aes)
	    tmp_ralink_cfg_enc="EncrypType=AES"
	;;
	tkip)
	    tmp_ralink_cfg_enc="EncrypType=TKIP"
	;;
	*)
	    log "Configuration not supported!" >&2
	    _error
	;;
    esac

    tmp_ralink_cfg_ssid="SSID=$wifi_ssid";
    tmp_ralink_cfg_psk="WPAPSK=$wifi_psk";
    tmp_ralink_cfg_def_key="DefaultKeyID=$wifi_wep_def_key";
    tmp_ralink_cfg_wep_key1="Key1Str=$wifi_wep_key1";
    tmp_ralink_cfg_wep_key2="Key2Str=$wifi_wep_key2";
    tmp_ralink_cfg_wep_key3="Key3Str=$wifi_wep_key3";
    tmp_ralink_cfg_wep_key4="Key4Str=$wifi_wep_key4";

    tmpKeyLen=`echo -n $wifi_wep_key1 | wc -c`
    if [ $tmpKeyLen == 5 ] || [ $tmpKeyLen == 13 ]; then
	tmp_ralink_wep_key1_type="Key1Type=1";
	tmp_ralink_wep_key2_type="Key2Type=1";
	tmp_ralink_wep_key3_type="Key3Type=1";
	tmp_ralink_wep_key4_type="Key4Type=1";
    else
	tmp_ralink_wep_key1_type="Key1Type=0";
	tmp_ralink_wep_key2_type="Key2Type=0";
	tmp_ralink_wep_key3_type="Key3Type=0";
	tmp_ralink_wep_key4_type="Key4Type=0";
    fi

    tmp_ralink_cfg_auth="";
    case $wifi_auth in
	open)
	    tmp_ralink_cfg_auth="AuthMode=OPEN"
	;;
	shared)
	    tmp_ralink_cfg_auth="AuthMode=SHARED"
	;;
	wep_auto)
	    tmp_ralink_cfg_auth="AuthMode=WEPAUTO"
	;;
	wpapsk)
	    tmp_ralink_cfg_auth="AuthMode=WPAPSK"
	;;
	wpa2psk)
	    tmp_ralink_cfg_auth="AuthMode=WPA2PSK"
	;;
	*)
	    log "Configuration not supported!" >&2
	    _error
	;;
    esac

    echo > $RALINK_CFG_FILE
    echo "Default" >> $RALINK_CFG_FILE
    echo "CountryRegion=5" >> $RALINK_CFG_FILE
    echo "CountryRegionABand=7" >> $RALINK_CFG_FILE
    echo "CountryCode=" >> $RALINK_CFG_FILE
    echo "ChannelGeography=1" >> $RALINK_CFG_FILE
    echo "$tmp_ralink_cfg_ssid" >> $RALINK_CFG_FILE
    echo "NetworkType=Infra" >> $RALINK_CFG_FILE
    echo "WirelessMode=9" >> $RALINK_CFG_FILE
    echo "Channel=0" >> $RALINK_CFG_FILE
    echo "BeaconPeriod=100" >> $RALINK_CFG_FILE
    echo "TxPower=100" >> $RALINK_CFG_FILE
      echo "BGProtection=0" >> $RALINK_CFG_FILE
      echo "TxPreamble=0" >> $RALINK_CFG_FILE
      echo "RTSThreshold=2347" >> $RALINK_CFG_FILE
      echo "FragThreshold=2346" >> $RALINK_CFG_FILE
      echo "TxBurst=1" >> $RALINK_CFG_FILE
      echo "PktAggregate=0" >> $RALINK_CFG_FILE
      echo "WmmCapable=1" >> $RALINK_CFG_FILE
      echo "AckPolicy=0;0;0;0" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_auth" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_enc" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_psk" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_def_key" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_wep_key1_type" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_wep_key1" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_wep_key2_type" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_wep_key2" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_wep_key3_type" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_wep_key3" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_wep_key4_type" >> $RALINK_CFG_FILE
      echo "$tmp_ralink_cfg_wep_key4" >> $RALINK_CFG_FILE
      echo "PSMode=CAM" >> $RALINK_CFG_FILE
      echo "AutoRoaming=0" >> $RALINK_CFG_FILE
      echo "RoamThreshold=70" >> $RALINK_CFG_FILE
      echo "APSDCapable=0" >> $RALINK_CFG_FILE
      echo "APSDAC=0;0;0;0" >> $RALINK_CFG_FILE
      echo "HT_RDG=1" >> $RALINK_CFG_FILE
      echo "HT_EXTCHA=0" >> $RALINK_CFG_FILE
      echo "HT_OpMode=1" >> $RALINK_CFG_FILE
      echo "HT_MpduDensity=4" >> $RALINK_CFG_FILE
      echo "HT_BW=1" >> $RALINK_CFG_FILE
      echo "HT_BADecline=0" >> $RALINK_CFG_FILE
      echo "HT_AutoBA=1" >> $RALINK_CFG_FILE
      echo "HT_BADecline=0" >> $RALINK_CFG_FILE
      echo "HT_AMSDU=0" >> $RALINK_CFG_FILE
      echo "HT_BAWinSize=64" >> $RALINK_CFG_FILE
      echo "HT_GI=1" >> $RALINK_CFG_FILE
      echo "HT_MCS=33" >> $RALINK_CFG_FILE
      echo "HT_MIMOPSMode=3" >> $RALINK_CFG_FILE
      echo "HT_DisallowTKIP=0" >> $RALINK_CFG_FILE
      echo "EthConvertMode=" >> $RALINK_CFG_FILE
      echo "EthCloneMac=" >> $RALINK_CFG_FILE
      echo "IEEE80211H=0" >> $RALINK_CFG_FILE
      echo "TGnWifiTest=0" >> $RALINK_CFG_FILE
      echo "WirelessEvent=0" >> $RALINK_CFG_FILE
      echo "MeshId=MESH" >> $RALINK_CFG_FILE
      echo "MeshAutoLink=1" >> $RALINK_CFG_FILE
      echo "MeshAuthMode=OPEN" >> $RALINK_CFG_FILE
      echo "MeshEncrypType=NONE" >> $RALINK_CFG_FILE
      echo "MeshWPAKEY=" >> $RALINK_CFG_FILE
      echo "MeshDefaultkey=1" >> $RALINK_CFG_FILE
      echo "MeshWEPKEY=" >> $RALINK_CFG_FILE
      echo "CarrierDetect=0" >> $RALINK_CFG_FILE
      echo "AntDiversity=0" >> $RALINK_CFG_FILE
      echo "BeaconLostTime=4" >> $RALINK_CFG_FILE
      echo "FtSupport=1" >> $RALINK_CFG_FILE
      echo "Wapiifname=ra0" >> $RALINK_CFG_FILE
      echo "WapiPsk=" >> $RALINK_CFG_FILE
      echo "WapiPskType=" >> $RALINK_CFG_FILE
      echo "WapiUserCertPath=" >> $RALINK_CFG_FILE
      echo "WapiAsCertPath=" >> $RALINK_CFG_FILE
      echo "PSP_XLINK_MODE=0" >> $RALINK_CFG_FILE
      echo "WscManufacturer=" >> $RALINK_CFG_FILE
      echo "WscModelName=" >> $RALINK_CFG_FILE
      echo "WscDeviceName=" >> $RALINK_CFG_FILE
      echo "WscModelNumber=" >> $RALINK_CFG_FILE
      echo "WscSerialNumber=" >> $RALINK_CFG_FILE
      echo "RadioOn=1" >> $RALINK_CFG_FILE
}

_check_variables()
{
  if [ -z "$wifi_driver" ]; then
    log "802.11 driver is not specified" >&2
    _error
  fi

  if [ -z "$wifi_ssid" ]; then
    wifi_ssid="default_ssid"
  fi

  if [ -z "$wifi_enc" ]; then
    wifi_enc="tkip"
  fi

  if [ -z "$wifi_auth" ]; then
    wifi_auth="wpa2psk"
  fi
}




# EXECUTION POINT

if [ -f $LOCK_FILE ]; then
  echo "Init is already running. Exiting..." >&2
  _error
fi

if [ -f $LOCK_ISACTIVE_FILE ]; then
  active_wifi_driver=`cat $LOCK_ISACTIVE_FILE`
  else active_wifi_driver=
fi

# make lock
echo -n > $LOCK_FILE
LOCK_OWNED=1

case $1 in
    conn_status)
          tmp_status=`iwpriv ra0 connStatus | awk '{ if (NR!=1) exit; line1=index($0,":"); if (line10=0) {print "Unknown"; exit}; line2=index($0,"("); if (line2==0) {len=length($0)-line1} else {len=line2-line1-1}; text=substr($0, line1+1, len); print text }'`
          if [ "$tmp_status" == "Connected" ]; then
            echo "Connected"
            else
              echo "Disconnected"
          fi
          _exit
    ;;
    get_ip)
      log "Fetching IP address info for wireless interface"

      #if [ ! -f $LOCK_ISACTIVE_FILE ]; then
	#echo "UNKNOWN"
	#_exit
      #fi	

      #wifi_driver=`cat $LOCK_ISACTIVE_FILE`
      #log "wifi_driver=$wifi_driver"

      ifconfig ra0 | grep 'inet addr' | awk '{ printf substr($2, 6) }'

      log "DONE."
      ;;
    status)
      if [ -f $LOCK_ISACTIVE_FILE ]; then
        echo "running"
      else
        echo "stopped"
      fi
    ;;
    stop)
        #if [ ! -f $LOCK_ISACTIVE_FILE ]; then
        #    _exit
        #fi

      log "Stopping driver"

      #wifi_driver=`cat $LOCK_ISACTIVE_FILE`
      #log "wifi_driver=$wifi_driver"

      read_nvram_env "wifi_int_ip"

      _make_ralink_deinit

      log "DONE."
      ;;
    restart)
	log "Restarting wifi..."

	$WIFI_STARTUP_SCRIPT stop
	$WIFI_STARTUP_SCRIPT start

        log "DONE."
	;;
    start)
	log "Start..."
        if [ -f $LOCK_ISACTIVE_FILE ]; then
	    _exit
        fi

      log "Starting wifi (driver=$wifi_driver)"

	read_nvram_env "wifi_ssid"
	read_nvram_env "wifi_psk"
	read_nvram_env "wifi_enc"
	read_nvram_env "wifi_auth"
	read_nvram_env "wifi_int_ip"
	read_nvram_env "wifi_int_mask"
	read_nvram_env "wifi_int_gw"
	read_nvram_env "wifi_int_dns"
	read_nvram_env "wifi_wep_def_key"
	read_nvram_env "wifi_wep_key1"
	read_nvram_env "wifi_wep_key2"
	read_nvram_env "wifi_wep_key3"
	read_nvram_env "wifi_wep_key4"
	
	log "ssid=${wifi_ssid}"
        #_check_variables

        sleep 1

        log "Ralink driver section"
        echo -n $wifi_driver > $LOCK_ISACTIVE_FILE
        log "Making configuration..."
        _make_ralink_cfg
        log "Initializing..."
        _make_ralink_init

        log "DONE."
    ;;
    *)
      echo "Wireless init script v1.0"
      echo "usage: [start|stop|status]"
      ;;
esac

_exit
